<?php
/**
 * $Horde: whups/search.php,v 1.117 2008/06/24 22:25:55 jan Exp $
 *
 * Copyright 2001-2002 Robert E. Coyle <robertecoyle@hotmail.com>
 * Copyright 2001-2008 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
 *
 * @author Chuck Hagenbuch <chuck@horde.org>
 */

/**
 * Construct the URL back to a supplied search
 */
function _getSearchUrl($vars)
{
    $qUrl = '';

    $queue = (int)$vars->get('queue');
    $qUrl = Util::addParameter($qUrl, array('queue' => $queue));

    $summary = $vars->get('summary');
    if ($summary) {
        $qUrl = Util::addParameter($qUrl, 'summary', $summary);
    }

    $states = $vars->get('states');
    if (is_array($states)) {
        foreach ($states as $type => $state) {
            if (is_array($state)) {
                foreach ($state as $s) {
                    $qUrl = Util::addParameter($qUrl, "states[$type][]", $s);
                }
            } else {
                $qUrl = Util::addParameter($qUrl, "states[$type]", $state);
            }
        }
    }

    return substr($qUrl, 1);
}

require_once dirname(__FILE__) . '/lib/base.php';
require_once WHUPS_BASE . '/lib/Query.php';
require_once WHUPS_BASE . '/lib/Forms/Search.php';
require_once 'Horde/Variables.php';
require_once WHUPS_BASE . '/lib/View.php';

$renderer = new Horde_Form_Renderer();
$beendone = 0;
$vars = Variables::getDefaultVariables();

$title = _("Search");
require WHUPS_TEMPLATES . '/common-header.inc';
require WHUPS_TEMPLATES . '/menu.inc';

// Update sorting preferences.
if (Util::getFormData('sortby') !== null) {
    $prefs->setValue('sortby', Util::getFormData('sortby'));
}
if (Util::getFormData('sortdir') !== null) {
    $prefs->setValue('sortdir', Util::getFormData('sortdir'));
}

$form = new SearchForm($vars);
if (($vars->get('formname') || $vars->get('queue') || $vars->get('summary') || $vars->get('states')) && $form->validate($vars, true)) {
    $form->getInfo($vars, $info);
    $tickets = $whups_driver->getTicketsByProperties($info);
    if (is_a($tickets, 'PEAR_Error')) {
        $notification->push(sprintf(_("There was an error performing your search: %s"), $tickets->getMessage()), 'horde.error');
    } else {
        Whups::sortTickets($tickets);

        $values = Whups::getSearchResultColumns();

        $_SESSION['whups']['last_search'] = Horde::applicationUrl('search.php?' . _getSearchUrl($vars));
        $results = Whups_View::factory('Results', array('title' => _("Search Results"),
                                                        'results' => $tickets,
                                                        'values' => $values,
                                                        'url' => $_SESSION['whups']['last_search']));
        $results->html();
        $beendone = 1;

        if (is_object($form)) {
            $form->setTitle(_("Refine Search"));
            $form->renderActive($renderer, $vars, 'search.php', 'get');
            echo '<br />';

            $beendone = 1;
        }
    }
}

if (!$beendone) {
    // Front search page.
    $form->setTitle(_("Ticket Search"));
    $form->renderActive($renderer, $vars, 'search.php', 'get');
    echo '<br class="spacer" />';
}

$qManager = new Whups_QueryManager();
$myqueries = Whups_View::factory(
    'SavedQueries',
    array('title' => Auth::getAuth() ? _("My Queries") : _("Public Queries"),
          'results' => $qManager->listQueries(Auth::getAuth())));
$myqueries->html();

require $registry->get('templates', 'horde') . '/common-footer.inc';
